package ika.proj;

import java.io.BufferedInputStream;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.Vector;

/**
 *
 * @author Bernhard Jenny, Institute of Cartography, ETH Zurich
 */
public class ProjectionsManager {

    private static Vector<String> list;

    private ProjectionsManager() {
    }

    public static int getProjectionID(String name) {
        if (ProjectionsManager.list == null) {
            loadProjectionsFromFile();
        }
        return list.indexOf(name);
    }

    public static String getProjectionName(int id) {
        if (ProjectionsManager.list == null) {
            loadProjectionsFromFile();
        }
        return list.get(id);
    }

    public static Vector<String> getProjectionNames() {
        if (ProjectionsManager.list == null) {
            loadProjectionsFromFile();
        }
        return list;
    }

    private static void loadProjectionsFromFile() {
        BufferedReader reader = null;
        try {
            list = new Vector<String>();

            // load projection description
            java.net.URL url = ProjectionsManager.class.getResource("/ika/data/projections.txt");
            BufferedInputStream bis = new BufferedInputStream(url.openStream());
            InputStreamReader isr = new InputStreamReader(bis, "UTF-8");
            reader = new BufferedReader(isr);
            String line;
            while ((line = reader.readLine()) != null) {
                String[] projDesc = line.split(",");
                list.add(projDesc[0]);
            }
        } catch (IOException e) {
            e.printStackTrace();
            list = null;
        } finally {
            if (reader != null) {
                try {
                    reader.close();
                } catch (IOException ex) {
                    ex.printStackTrace();
                }
            }
        }
    }

}
